// PhysPara_BPM_SC_T.ino
// ESP32 @ IDE 1.8.12

#include <OneWire.h>
#include <DallasTemperature.h>

#define pulseLED 2

// BPM parameter
int BPMpin = 34;
unsigned long duration, t0 ;
unsigned int pulseMin = 30, pulseMax = 180;
float pulseRate, pulseRate_old, tol = 8;

// T & SC parameter
const int oneWireBus = 14;     
const int conductPin = 35;
float temperatureC;
float Tfact = 10.0;
int conductVal;
float conductFact = 0.04;

OneWire oneWire(oneWireBus);
DallasTemperature sensors(&oneWire);

void setup() 
{ Serial.begin(250000);
  pinMode(BPMpin, INPUT);
  pinMode(pulseLED, OUTPUT);
  sensors.begin();
  Serial.print("heart_rate_BpM temperature_ºC skin_conductivity");
}

void loop() 
{ // get pulse rate
  duration = 0;
  t0 = millis();
  while(digitalRead(BPMpin) == 1) 
  { digitalWrite(pulseLED, HIGH); delay(10);
  } 
    
  while(digitalRead(BPMpin) == 0) 
  { digitalWrite(pulseLED, LOW); delay(10);
  }
  duration = millis() - t0;
  pulseRate = 60000.0/duration;     // in BPM: 60*1000 ms = 60 s
  
  // get skin temperature
  sensors.requestTemperatures(); 
  temperatureC = sensors.getTempCByIndex(0);

  // get skin conductivity
  conductVal = analogRead(conductPin);
    
  if ((pulseRate >= pulseMin) && (pulseRate <= pulseMax) && (abs(pulseRate - pulseRate_old) < tol))
  { Serial.print(pulseRate);  Serial.print(", ");  
    // Serial.print(pulseMin);   Serial.print(", ");  
    // Serial.print(pulseMax);   Serial.print(", ");
        
    Serial.print(Tfact*(temperatureC-30)+60); Serial.print(", ");
    Serial.println(conductFact*conductVal);
  } 
  pulseRate_old = pulseRate;
}
